#
# menu class
# menu.py
#

import pygame
from pygame import *
import world
import highscore

class menu():
	def __init__(self,screen,resolution):
		self._screen = screen
		self._should_quit = False
		self._res = resolution
		self._font = pygame.font.SysFont("courbd", 64);
		self._levels = ["level1","level2"]
		self._cur_level = 0
		self._last_score = 0
		self._pages = {
		"main": {"background": pygame.image.load("menu.png").convert()},
		"instructions": {"background": pygame.image.load("instructions.png").convert()},
		"highscore": {"background": pygame.image.load("highscore.png").convert()},
		"won": {"background": pygame.image.load("won.png").convert()},
		"lost": {"background": pygame.image.load("lost.png").convert()},
		}
		self._curpage = "main"
		self._state = "menu"
		self._menu_options = [
			{"text":"Start Game","img": self._menu_option_img("Start Game"),
			"selimg": self._menu_option_img("Start Game",True)},
			{"text":"Instructions","img": self._menu_option_img("Instructions"),
			"selimg": self._menu_option_img("Instructions",True)},
			{"text":"Highscore","img": self._menu_option_img("Highscore"),
			"selimg": self._menu_option_img("Highscore",True)},
			{"text":"Exit","img": self._menu_option_img("Exit"),
			"selimg": self._menu_option_img("Exit",True)},
			]
		# highscore
		self._hs = highscore.highscore(self._screen,self._res)
	
	def _menu_option_img(self,text,selected = False):
		black = self._font.render(text,True,(0,0,0)).convert_alpha()
		white = self._font.render(text,True,(255,255,255)).convert_alpha()
		if not selected:
			blue = self._font.render(text,True,(128,128,255)).convert_alpha()
		else:
			selblue = self._font.render(text,True,(200,200,255)).convert_alpha()
		sf = pygame.Surface((black.get_width() + 4,black.get_height() + 4),0,32).convert_alpha()
		sf.fill((0,0,0,0))
		sf.blit(white,(0,0))
		sf.blit(black,(4,4))
		if not selected:
			sf.blit(blue,(2,2))
		else:
			sf.blit(selblue,(2,2))
		return sf
	
	def process_event(self,event):
		if self._state == "game":
			if event.type == MOUSEBUTTONDOWN:
				self._wo.process_mousedown(event)
			if event.type == KEYDOWN:
				if event.key == K_ESCAPE:
					self._curpage = "main"
					self._state = "menu"
					del self._wo
		elif self._state == "menu":
			if event.type == MOUSEBUTTONDOWN:
				if event.button == 1:
					self._clicked(event.pos)
		elif self._state == "enterscore":
			if event.type == KEYDOWN:
				if self._hs.process_key_event(event):
					# done writing name switching back to regular highscore
					self._state = "menu"
			if event.type == MOUSEBUTTONDOWN:
				if event.button == 1:
					# faking an event...
					self.key = K_RETURN
					if self._hs.process_key_event(self):
						# done writing name switching back to regular highscore
						self._state = "menu"
			

	def process(self,time_passed):
		if self._state == "game":
			res = self._wo.process(time_passed)
			if res != None:
				self._last_score = res[1]
				self._state = "menu"
				if res[0] == world.WON:
					self._curpage = "won"
				else:
					self._curpage = "lost"
		if self._should_quit:
			return False
		return True
		
	def draw(self):
		if self._state == "game":
			self._wo.draw()
		else:
			self._draw_page()
			
	def _draw_page(self):
		self._screen.blit(self._pages[self._curpage]["background"],(0,0))
		if self._curpage == "main":
			# draw options
			yoff = (self._res[1] - len(self._menu_options) * 70) / 2
			xoff = self._res[0] / 2
			mpos = pygame.mouse.get_pos()
			for mo in self._menu_options:
				img = mo["img"]
				if mo["img"].get_rect().collidepoint(mpos[0] - (xoff - img.get_width() / 2),mpos[1] - yoff):
					img = mo["selimg"]
				self._screen.blit(img,(xoff - img.get_width() / 2,yoff))
				yoff += 70
		if self._curpage == "highscore" and self._state == "menu":
			self._hs.draw_show()
		if self._curpage == "highscore" and self._state == "enterscore":
			self._hs.draw_enter()
			
	
	def _clicked(self,mpos):
		if self._curpage == "main":
			yoff = (self._res[1] - len(self._menu_options) * 70) / 2
			xoff = self._res[0] / 2
			clicked_option = None
			for mo in self._menu_options:
				img = mo["img"]
				if mo["img"].get_rect().collidepoint(mpos[0] - (xoff - img.get_width() / 2),mpos[1] - yoff):
					clicked_option = mo["text"]
				yoff += 70
			if clicked_option == "Start Game":
				self._cur_level = 0
				self._wo = world.world(self._screen,self._res,self._levels[self._cur_level],0)
				self._state = "game"
				return
			if clicked_option == "Instructions":
				self._curpage = "instructions"
				return
			if clicked_option == "Highscore":
				self._curpage = "highscore"
				return
			if clicked_option == "Exit":
				self._should_quit = True
				return
		if self._curpage == "instructions":
			self._curpage = "main"
			return
		if self._curpage == "highscore":
			self._curpage = "main"
			return
		if self._curpage == "lost":
			del self._wo
			if self._hs.is_highscore(self._last_score):
				self._hs.new_highscore(self._last_score)
				self._state = "enterscore"
				self._curpage = "highscore"
				return
			self._curpage = "main"
			return
		if self._curpage == "won":
			del self._wo
			self._cur_level += 1
			if self._cur_level >= len(self._levels):
				if self._hs.is_highscore(self._last_score):
					self._hs.new_highscore(self._last_score)
					self._state = "enterscore"
					self._curpage = "highscore"
					return				
				self._curpage = "main"
				return
			self._wo = world.world(self._screen,self._res,self._levels[self._cur_level],self._last_score)
			self._state = "game"
			return
