#
# shot class
# shot.py
#

import math
import pygame
from pygame.locals import *
import gameobj
import vector

SIMPLE = 1
POWER = 2
GRENADE = 3
GRENADE_PARTS = 32
SHOT_DICT = {
# Type: ["filename.png",speed,dist,damage,loadedsurface]
1: ["simpleshot.png",200,250,9,None],
2: ["powershot.png",200,250,49,None],
3: ["grenadeshot.png",100,200,0,None]
}

def load_shots():
	global SHOT_DICT
	for i in SHOT_DICT:
		SHOT_DICT[i][4] = pygame.image.load(SHOT_DICT[i][0]).convert_alpha()

class shot(gameobj.gameobj):
	def __init__(self,world,type,raw_pos,raw_heading):
		global SHOT_DICT
		gameobj.gameobj.__init__(self,world)
		self._owner = None
		self._type = type
		self._speed = SHOT_DICT[type][1]
		self._dist = SHOT_DICT[type][2]
		if self._type == GRENADE:
			self._level_collision = False
			self._dist = raw_heading.len()
			self._speed = self._speed * (self._dist / SHOT_DICT[type][2])
		self._startdist = self._dist
		self._dmg = SHOT_DICT[type][3]
		self.size = 8
		if self._type == SIMPLE:
			self.size = 4
		self._sprite = SHOT_DICT[type][4]
		self._orig_sprite = self._sprite
		self._draw_offset = vector.twin(self.size / 2.0, self.size / 2.0)
		self._pos = raw_pos
		self._heading = vector.twin(raw_heading.get())
		self._heading.normalize()
	
	def draw(self,pos):
		self._world._screen.blit(self._sprite,(pos - self._draw_offset).get())
	
	def set_id(self,id):
		self._id = id
	
	def set_owner(self,owner):
		self._owner = owner
	
	def process(self,time_passed):
		gameobj.gameobj.process(self,time_passed)
		# normal movement
		self._dist -= (time_passed * self._speed) / 1000.0
		if self._dist <= 0:
			# if its a grenade we destroy it and create alot of simpe shots around it in a circle
			if self._type == GRENADE:
				self._create_grenade_blast()
			self._world.del_obj(self._id)
		# if grenade scale the shot image
		if self._type == GRENADE:
			halfdist = self._startdist / 2.0
			nw = self._orig_sprite.get_width() * (2.0 - (abs(halfdist - self._dist)) / halfdist)
			nh = self._orig_sprite.get_height() * (2.0 - (abs(halfdist - self._dist)) / halfdist)
			self._draw_offset = vector.twin(nw / 2.0, nh / 2.0)
			self._sprite = pygame.transform.scale(self._orig_sprite,(int(nw),int(nh)))
	
	def collided(self,time_passed):
		self._world.del_obj(self._id)
		
	def _create_grenade_blast(self):
		angle = 0.0
		while angle < 360:
			id = self._world.add_obj(shot(self._world,SIMPLE,
				self._pos,
				vector.twin(math.cos(math.radians(angle)),math.sin(math.radians(angle)))))
			self._world._objs[id].set_id(id)
			self._world._objs[id].set_owner(self._owner)
			angle += 360.0 / GRENADE_PARTS
